"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateApolloClient = exports.generateHeaders = void 0;
const client_1 = require("@apollo/client");
const utilities_1 = require("@apollo/client/utilities");
const apollo_link_1 = require("apollo-link");
const apollo_link_ws_1 = require("apollo-link-ws");
const node_fetch_1 = __importDefault(require("node-fetch"));
const path_1 = __importDefault(require("path"));
const debug_1 = require("./debug");
const react_apollo_network_status_1 = require("react-apollo-network-status");
const moduleLog = debug_1.debug.extend('client');
let ws;
if (typeof (window) !== 'object') {
    ws = require('ws');
}
const DEEP_FOUNDATION_HASURA_RELATIVE = ((r) => r ? !!+r : undefined)(process.env.DEEP_FOUNDATION_HASURA_RELATIVE);
const NEXT_PUBLIC_DEEP_FOUNDATION_HASURA_RELATIVE = ((r) => r ? !!+r : undefined)(process.env.NEXT_PUBLIC_DEEP_FOUNDATION_HASURA_RELATIVE);
const ENV_RELATIVE = typeof (DEEP_FOUNDATION_HASURA_RELATIVE) === 'boolean' ? DEEP_FOUNDATION_HASURA_RELATIVE : typeof (NEXT_PUBLIC_DEEP_FOUNDATION_HASURA_RELATIVE) === 'boolean' ? NEXT_PUBLIC_DEEP_FOUNDATION_HASURA_RELATIVE : undefined;
function generateHeaders(options) {
    const headers = Object.assign({}, options.headers);
    if (options.token)
        headers.Authorization = `Bearer ${options.token}`;
    if (options.secret)
        headers['x-hasura-admin-secret'] = options.secret;
    if (options.client)
        headers['x-hasura-client'] = options.client;
    return headers;
}
exports.generateHeaders = generateHeaders;
const host = typeof (window) === 'object' ? window.location.host : '';
function generateApolloClient(options, forwardingArguments) {
    var _a, _b, _c, _d, _e;
    const log = moduleLog.extend(generateApolloClient.name);
    log({ options, forwardingArguments });
    const isRelative = typeof (options === null || options === void 0 ? void 0 : options.relative) === 'boolean' ? options.relative : typeof (ENV_RELATIVE) === 'boolean' ? ENV_RELATIVE : false;
    const headers = generateHeaders(options);
    const httpLink = new client_1.HttpLink({
        uri: `${isRelative ? '' : `http${options.ssl ? 's' : ''}:/`}${path_1.default.normalize('/' + (options.path || ''))}`,
        fetch: node_fetch_1.default,
        headers,
    });
    const wsLink = options.ws
        ? new apollo_link_ws_1.WebSocketLink({
            uri: `${isRelative ? (host ? `ws${options.ssl ? 's' : ''}://${host}` : '') : `ws${options.ssl ? 's' : ''}:/`}${path_1.default.normalize('/' + (options.path || ''))}`,
            options: {
                lazy: true,
                reconnect: true,
                connectionParams: () => ({
                    headers,
                }),
            },
            webSocketImpl: ws,
        })
        : null;
    const authMiddleware = new apollo_link_1.ApolloLink((operation, forward) => {
        operation.setContext({
            headers,
        });
        return forward(operation);
    });
    const link = !options.ws
        ? httpLink
        : (0, apollo_link_1.split)(({ query }) => {
            const def = (0, utilities_1.getMainDefinition)(query);
            return (def === null || def === void 0 ? void 0 : def.kind) === 'OperationDefinition' && (def === null || def === void 0 ? void 0 : def.operation) === 'subscription';
        }, wsLink, httpLink);
    const { link: notifierLink, useApolloNetworkStatus } = (0, react_apollo_network_status_1.createNetworkStatusNotifier)();
    const client = new client_1.ApolloClient(Object.assign(Object.assign({ ssrMode: true, link: (0, apollo_link_1.concat)(notifierLink, (0, apollo_link_1.concat)(authMiddleware, link)), connectToDevTools: true, cache: new client_1.InMemoryCache(Object.assign(Object.assign({}, forwardingArguments === null || forwardingArguments === void 0 ? void 0 : forwardingArguments.InMemoryCache), { freezeResults: false, resultCaching: false })).restore(options.initialStore || {}) }, forwardingArguments === null || forwardingArguments === void 0 ? void 0 : forwardingArguments.ApolloClient), { defaultOptions: Object.assign({ watchQuery: Object.assign({ fetchPolicy: 'no-cache' }, (_b = (_a = forwardingArguments === null || forwardingArguments === void 0 ? void 0 : forwardingArguments.ApolloClient) === null || _a === void 0 ? void 0 : _a.defaultOptions) === null || _b === void 0 ? void 0 : _b.watchQuery), query: Object.assign({ fetchPolicy: 'no-cache' }, (_d = (_c = forwardingArguments === null || forwardingArguments === void 0 ? void 0 : forwardingArguments.ApolloClient) === null || _c === void 0 ? void 0 : _c.defaultOptions) === null || _d === void 0 ? void 0 : _d.query) }, (_e = forwardingArguments === null || forwardingArguments === void 0 ? void 0 : forwardingArguments.ApolloClient) === null || _e === void 0 ? void 0 : _e.defaultOptions) }));
    client.jwt_token = options.token;
    client.path = options.path;
    client.ssl = options.ssl;
    client.useApolloNetworkStatus = useApolloNetworkStatus;
    log({ client });
    return client;
}
exports.generateApolloClient = generateApolloClient;
//# sourceMappingURL=client.js.map